pro caxes, cmpl, cx, cy, ymag

  nc = n_elements(cmpl(0,*))
  jc = 0
  tmpy = fltarr(nc)
  tmpx = fltarr(nc)
  tol = 1e-3

  ; Find the unique y-values
  tmpy[0] = cmpl[ymag,0]
  tmpx[0] = cmpl[0,0] - cmpl[1,0]
  ny = 1
  nx = 1
  for i=0,nc-1 do begin
    y = cmpl[ymag, i] 
    x = cmpl[0,i] - cmpl[1,i]
    wy = where(abs(tmpy[0:ny-1] - y) lt tol)
    wx = where(abs(tmpx[0:ny-1] - x) lt tol)
    if (wy[0] eq -1) then begin
      tmpy[ny] = y
      ny = ny+1
    end
    if (wx[0] eq -1) then begin
      tmpx[nx] = x
      nx = nx+1
    end
  end

  cy0 = tmpy[0:ny-1]
  cx0 = tmpx[0:nx-1]
  scy = sort(cy0)
  scx = sort(cx0)
  cy = cy0[scy]
  cx = cx0[scx]

end

; ------------------------------------------------------------------

function mkcarr,cx, cy, cmpl, ymag

  ncx = n_elements(cx)
  ncy = n_elements(cy)
  carr = fltarr(ncx, ncy)
  tol = 1e-3

; First pass: fill in the fields that have data and mark the others with -1

  for iy=0,ncy-1 do begin
    for ix=0,ncx-1 do begin
      w = where((abs(cmpl[ymag,*] - cy[iy]) lt tol) AND abs((cmpl[0,*]-cmpl[1,*]) - cx[ix]) lt tol)
      if (w[0] ge 0) then $
        carr[ix,iy] = cmpl[2,w[0]] $
      else $
        carr[ix,iy] = -1
    end
  end

  print, '--------'
  print, carr
  print, '--------'

; Second pass: interpolate the fields marked with -1 in the first pass

  for iy=0,ncy-1 do begin
    for ix=0,ncx-1 do begin
      if (carr[ix,iy] eq -1) then begin
        wok = where(carr[*,iy] ge 0)
	nwok = n_elements(wok)
        if (cx[ix] lt min(cx[wok])) then $
	  carr[ix,iy] = carr[wok[0],iy] $
        else if (cx[ix] gt max(cx[wok])) then $
          carr[ix,iy] = carr[wok[nwok-1],iy] $
        else $
	  carr[ix,iy] = interpol(carr[wok,iy], cx[wok], cx[ix])
      end
    end
  end

  return, carr

end

; ------------------------------------------------------------------

function clookup, cx, cy, carr, x, y
  ncx = n_elements(cx)
  ncy = n_elements(cy)
  ix = interpol(findgen(ncx), cx, x)
  iy = interpol(findgen(ncy), cy, y)
  c = interpolate(carr, ix, iy)
;  print, x, y, ix, iy, c
  return, c
end

; ------------------------------------------------------------------

function cfill,cx,cy,carr, nx, ny, xr, yr

  dx = 1.0*(xr[1] - xr[0])
  dy = 1.0*(yr[1] - yr[0])
  cpixels = fltarr(nx, ny)

  for iy=0,ny-1 do begin
    y = yr[0] + iy*dy/ny
    for ix=0,nx-1 do begin
      x = xr[0] + ix*dx/nx
      cpixels[ix,iy] = clookup(cx, cy, carr, x, y)
    end
  end

  return, cpixels

end

; ------------------------------------------------------------------

function cmplarr, cmpl, nx, ny, xr, yr, ymag=ymag

  if (n_elements(ymag) eq 0) then ymag = 0

; Populate a 2-D array with numbers between 0 and 1, indicating the
; detection completeness as a function of colour and magnitude

  print,'nx, ny, xr, yr = ',nx,ny,xr,yr

  caxes, cmpl, cx, cy, ymag
  print, 'Unique y-values = ',cy
  print, 'Unique x-values = ',cx

  carr = mkcarr(cx, cy, cmpl, ymag)

  print, carr

  cpixels = cfill(cx,cy,carr, nx, ny, xr, yr)

  return, cpixels

end
