pro addkernel, kernel, pixels, x, y, weight
  sk = size(kernel)
  if (sk[0] eq 1) then begin
    kdx = sk[1]
    kdy = sk[1]
  end else begin
    kdx = sk[1]
    kdy = sk[2]
  end

  sp = size(pixels)
  pdx = sp[1]
  pdy = sp[2]

  px1 = fix(x+0.5) - kdx/2
  px2 = fix(x+0.5) + kdx/2
  py1 = fix(y+0.5) - kdy/2
  py2 = fix(y+0.5) + kdy/2
;  print, px1,px2,py1,py2, pdx, pdy

  if (px2 lt 0 OR py2 lt 0 OR px1 ge pdx OR py1 ge pdy) then return

  kx1 = 0
  kx2 = kdx-1
  ky1 = 0
  ky2 = kdy-1


  if (px1 lt 0) then begin
    kx1 = -px1
    px1 = 0
  end
  if (py1 lt 0) then begin
    ky1 = -py1
    py1 = 0
  end

  if (px2 ge pdx) then begin
    kx2 = kdx-1-(px2-pdx+1)
    px2 = pdx-1
  end
  if (py2 ge pdy) then begin
    ky2 = kdy-1-(py2-pdy+1)
    py2 = pdy-1
  end

;  print, px1,px2,py1,py2
;  print, kx1,kx2,ky1,ky2, kdx, kdy
  pixels[px1:px2,py1:py2] = pixels[px1:px2,py1:py2]  $
                          + weight*kernel[kx1:kx2,ky1:ky2]
end

; ============================================================================

function err_ellipse, em1, em2, ymag, xstep, ystep, dxmin, dxmax, dymin, dymax

; print, em1, em2, ymag, xstep, ystep, dxmin, dxmax, dymin, dymax

  if (em1 lt 0.0001) then em1 = 0.0001
  if (em2 lt 0.0001) then em2 = 0.0001

  nx = 5 * sqrt(em1*em1+em2*em2) / abs(xstep)

  if (ymag eq 0) then $
    ny = 5 * em1 / abs(ystep) $
  else $
    ny = 5 * em2 / abs(ystep)

  if (nx lt dxmin) then nx = dxmin
  if (nx gt dxmax) then nx = dxmax
  if (ny lt dymin) then ny = dymin
  if (ny gt dymax) then ny = dymax

  nx = fix(nx/2)*2+1          ; Number of pixels we aim for in the
  ny = fix(ny/2)*2+1          ; output kernel. 
  xdim = nx * xstep
  ydim = ny * ystep
  nycp = fix(abs(nx*ydim/xdim)) ; Number of pixels we'll need to copy to
                                ; get ny in the output kernel

  if (nx lt 3) then nx = 3
  if (ny lt 3) then ny = 3

  nx2 = nx - 2

  k1tmp = fltarr(nx, nx)     ; Work on a quadratic kernel internally

  xx = (findgen(nx) - nx/2) * abs(xstep)
  xx2 = (findgen(nx2) - nx2/2) * abs(xstep)

  fm1 = 0.0 * xx
  fm2 = 0.0 * xx2

  w1 = where(abs(xx) lt 3*em1)
  w2 = where(abs(xx2) lt 3*em2)

  fm1[w1] = exp(-xx[w1]*xx[w1] / (2*em1*em1))
  fm2[w2] = exp(-xx2[w2]*xx2[w2] / (2*em2*em2))

  if (ymag eq 0) then $
    for i=0,nx-1 do begin
      k1tmp[i,nx-i-1] = fm1[i]
      if (i gt 0 AND i lt nx-1) then begin
        k1tmp[i,nx-i] = 0.5*fm1[i]
        k1tmp[i,nx-i-2] = 0.5*fm1[i]
      end
    end $
  else $
    for i=0,nx-1 do begin
      k1tmp[i,i] = fm1[i]
      if (i gt 0 AND i lt nx-1) then begin
        k1tmp[i,nx-i] = 0.5*fm1[i]
        k1tmp[i,nx-i-2] = 0.5*fm1[i]
      end
    end

;  tvimage, (!D.TABLE_SIZE-1)*k1tmp/max(k1tmp), /nointerpolation
    
;  k3tmp = convol(k1tmp, fm2, /edge_truncate)

  ysum = fltarr(nx)
  if (nycp lt nx) then begin
    y0 = (nx-nycp)/2
    ysum = total(k1tmp[*,y0:y0+nycp-1], 2)
  endif else begin
    y0 = (nycp-nx)/2
    ysum = total(k1tmp[*,*], 2)
  endelse

  k1reb = fltarr(nx, ny)
;  print, nx,ny,nycp

  for x=0,nx-1 do begin
    if (ymag eq 0) then $
      yreb = (ny/2) - (x-nx/2)*1.0*abs(xstep/ystep) $
    else $
      yreb = (ny/2) + (x-nx/2)*1.0*abs(xstep/ystep)

    if (yreb ge 0 AND yreb lt ny-1) then begin
      y1 = fix(yreb)
      y2 = fix(yreb)+1
      dy = yreb - y1
      k1reb[x, y1] = ysum[x]*(1-dy)
      k1reb[x, y2] = ysum[x]*dy
    end
  end

  kernel = convol(k1reb, fm2, /edge_truncate)

;  kernel = congrid(k3, nx, ny, /interp, /minus_one)

;  tvimage, (!D.TABLE_SIZE-1)*kernel/max(kernel), /nointerpolation

  return, kernel

end

; ============================================================================

function calckernel, ktype, kwidth

  case ktype of
    'GAUSS' : begin
                kdim = 1+2*fix(kwidth * 2.5)
                kdist = fltarr(kdim,kdim)
                for yk=0,kdim-1 do for xk=0,kdim-1 do $ 
                  kdist[xk,yk] = sqrt( (xk-kdim/2)^2. + (yk-kdim/2)^2.)
                kernel = exp(-kdist^2 / (2*kwidth^2))
              end
    'SQUARE': begin
                kdim = 2*fix(kwidth+0.5)+1
		if (kdim lt 3) then kdim = 3
                kernel = fltarr(kdim,kdim)+1
              end
    'DISC'  : begin
                kdim = 1+2*fix(kwidth + 0.5)
		if (kdim lt 3) then kdim = 3
                kdist = fltarr(kdim,kdim)
                for yk=0,kdim-1 do for xk=0,kdim-1 do $ 
                  kdist[xk,yk] = sqrt( (xk-kdim/2)^2. + (yk-kdim/2)^2.)
		w = where(kdist le kwidth)
                kernel = fltarr(kdim,kdim)
		kernel[w] = 1.
              end
     'DELTA' : begin
		kernel = fltarr(3,3)
		kernel[1,1] = 1.
              end
    else    : print, '** HESS_SFH: Unknown kernel: ',ktype
  end

;  print, 'Kernel dimension = ',kdim
  return, kernel

end

; ============================================================================

pro hess_sfh, m1, m2, weights=weights, xr=xr, yr=yr, $
  nx=nx, ny=ny, logscl=logscl, sigker=sigker, cont=cont, clevels=clevels, $
  color=color, neg=neg, _Extra=extra, $
  pixels=pixels, xp=xp, yp=yp, noshow=noshow, $
  ymag=ymag, errm1=errm1, errm2=errm2, cpixels=cpixels, cmode=cmode, $
  ktype=ktype

; HESS diagram generator/plotter modified for use with SFH.
; Instead of a generic tool to plot Y vs X, this version plots
; m1-m2 vs m[ymag]. This is necessary in order to allow realistic
; simulation of the error ellipse.
; The arrays errm1 and errm2 are two-dimensional arrays specifying 
; the mean error vs magnitude for m1 and m2.
; errm1[0,*] = magnitudes
; errm1[1,*] = errors
; These arrays will be interpolated during the actual plotting

  if (n_elements(weights) eq 0) then weights=fltarr(n_elements(m1))+1.
  if (n_elements(xr) eq 0) then xr = [min(m1-m2), max(m1-m2)]
  if (n_elements(ymag) eq 0) then ymag=0
  if (ymag eq 0) then my=m1 else my=m2
  if (n_elements(yr) eq 0) then yr = [max(my), min(my)]
  if (n_elements(nx) eq 0) then nx = 200
  if (n_elements(ny) eq 0) then ny = 200
  if (n_elements(clevels) eq 0) then clevels = findgen(10)/10
  if (n_elements(cmode) eq 0) then cmode = 'BEFORE'
  if (n_elements(ktype) eq 0) then ktype = 'GAUSS'

  if (n_elements(noshow) eq 0) then $
    plot, xr, yr, psym=3, xtitle=xtitle, ytitle=ytitle, xr=xr, yr=yr, $
      xstyle=1, ystyle=1, /nodata, color=color, _extra=extra

  dx = 1.0*(xr[1] - xr[0])
  dy = 1.0*(yr[1] - yr[0])
  xstep = dx/nx
  ystep = dy/ny

  if (n_elements(sigker) eq 0) then sigker=0.010*dx

; Define the kernel

  sigker_p = sigker / xstep
  kernel = calckernel(ktype,sigker_p)

; Draw the Hess diagram by adding the kernel at each data point

  if (n_elements(cpixels) gt 0) then begin
    if (cmode eq 'BEFORE') then $
      print,"HESS_CMD: Completeness correction BEFORE error broadening" $
    else $
      print,"HESS_CMD: NO Completeness correction BEFORE error broadening"
  end

  emy1 = 0.
  emy2 = 0.
  if (n_elements(errm1) gt 0) then begin
    if (ymag eq 0) then errmy = errm1 else errmy = errm2
    emy1 = 2*interpol(errmy(1,*), errmy(0,*), min(yr))
    emy2 = 2*interpol(errmy(1,*), errmy(0,*), max(yr))
  end

  pixels = fltarr(nx,ny)
  xp = xr[0] + findgen(nx)*(xr[1]-xr[0])/nx
  yp = yr[0] + findgen(ny)*(yr[1]-yr[0])/ny
  w = where((m1-m2) gt min(xr) AND (m1-m2) lt max(xr) AND $
               my gt min(yr)-emy1 AND my lt max(yr)+emy2)

  if (w[0] eq -1) then begin
    print, 'Warning - no data inside plot window'
    return
  end

  mdiff = m1-m2
  if (n_elements(errm1) gt 0) then begin
    em1 = interpol(errm1(1,*), errm1(0,*), m1)
    em2 = interpol(errm2(1,*), errm2(0,*), m2)

    w1 = where(m1 lt min(errm1(0,*)))
    w2 = where(m2 lt min(errm2(0,*)))
    if (w1[0] ge 0) then em1[w1] = min(errm1(1,*))
    if (w2[0] ge 0) then em2[w2] = min(errm2(1,*))

    w1 = where(m1 gt max(errm1(0,*)))
    w2 = where(m2 gt max(errm2(0,*)))
    if (w1[0] ge 0) then em1[w1] = max(errm1(1,*))
    if (w2[0] ge 0) then em2[w2] = max(errm2(1,*))
  end

  for i=0L,n_elements(w)-1 do begin
    if (n_elements(errm1) gt 0) then begin
      s = size(kernel)
      ekernel = err_ellipse(em1[w[i]], em2[w[i]], ymag, xstep, ystep, $
                 s[1]+2, nx/2, s[2]+2, ny/2)
      k_cnv = convol(ekernel,kernel,/edge_truncate)
;      k_cnv = ekernel
    end else begin
      k_cnv = kernel
    endelse
    px = nx*(mdiff[w[i]]-xr[0])/dx
    py = ny*(my[w[i]]-yr[0])/dy

; Take the incompleteness array into account, if present and cmode='BEFORE'
    if ((n_elements(cpixels) gt 0) AND (cmode eq 'BEFORE')) then begin
      csz = size(cpixels)
      cx = fix(px)
      cy = fix(py)
      if (cx lt 0) then cx = 0
      if (cy lt 0) then cy = 0
      if (cx ge csz[1]) then cx = csz[1]-1
      if (cy ge csz[2]) then cy = csz[2]-1
      wcompl = cpixels[cx, cy]
    end else $
      wcompl = 1.0

    addkernel, k_cnv/total(k_cnv), pixels, px, py, weights[w[i]] * wcompl
  end

  if (n_elements(logscl) gt 0) then begin
    pp = alog10(pixels/max(pixels)+logscl)
    pscl = (!D.TABLE_SIZE-1)*(pp-min(pp))/(max(pp)-min(pp))
  end else $
    pscl = (!D.TABLE_SIZE-1)*(pixels/max(pixels))

  if (n_elements(neg) gt 0) then pscl = max(pscl)-pscl
  if (n_elements(noshow) eq 0) then begin
    tvimage, pscl, /overplot

  ; We have to redraw the axes

    axis, xaxis=0, xstyle=1, xtickformat='(A1)', color=color
    axis, xaxis=1, xstyle=1, xtickformat='(A1)', color=color
    axis, yaxis=0, ystyle=1, ytickformat='(A1)', color=color
    axis, yaxis=1, ystyle=1, ytickformat='(A1)', color=color

  ; Contours

    if (n_elements(cont) gt 0) then begin
      xx = xr[0] + dx*findgen(nx)/nx
      yy = yr[0] + dy*findgen(ny)/ny
      contour, pixels/max(pixels), xx, yy, /overplot, levels=clevels
    end
  end

end
