; Scale a number of Hess diagrams for individual isochrones to a general
; star formation history.

pro sfhcmd, loga, hpixels, sfr=sfr, pixels=pixels, cpixels=cpixels, cmode=cmode

; loga contains the age bins.
; hpixels is the array of Hess diagrams for each age and metallicity.
;         This must have [n_elements(loga) * cb_nz] planes
; If produced by isocmd.pro, these will be normalised to a total mass of
; one Msun. 

@cb_sfh.jou

  nloga = n_elements(loga)
  if (n_elements(sfr) eq 0) then sfr = fltarr(nloga*cb_nz)+1. ; Default is uniform SFR

  pixels = hpixels[0,*,*] * 0.0
  k = 0
  for j=0,cb_nz-1 do begin
    for i=0,nloga-1 do begin
  ; "Old" approach: The i'th isochrone covers the age interval loga[i]..loga[i+1]
  ;    if (i lt nloga-1) then da = 10^(loga[i+1]-6.0) - 10^(loga[i]-6.0)
  ; New approach: the i'th isochrone covers the age interval
  ;   (loga[i-1]+loga[i])/2 .. (loga[i]+loga[i+1])/2

      if (i eq 0) then $
	loga1 = min([loga[i],cb_bins]) $
      else $
	loga1 = (loga[i-1] + loga[i]) / 2.

      if (i eq nloga-1) then $
	loga2 = max([loga[i],cb_bins]) $
      else $
	loga2 = (loga[i] + loga[i+1]) / 2.

      da = 10^(loga2 - 6.0) - 10^(loga1 - 6.0)
      pixels = pixels + hpixels[k,*,*] * da * sfr[k]
      k = k+1
    end
  end

  if (n_elements(cpixels) gt 1) then begin
    if (cmode eq 'AFTER') then begin
      print, 'SFHCMD: Completeness correction AFTER error broadening'
      pixels = pixels*cpixels
    endif
  end else begin
    print, 'SFHCMD: NO completeness correction AFTER error broadening'
  endelse

end
